/*! normalize.css v2.1.3 | MIT License | git.io/normalize */
/* Compressed with: yui-compressor --line-break 1 normalize.css */

article,aside,details,figcaption,figure,footer,header,hgroup,main,nav,section,summary{display:block}
audio,canvas,video{display:inline-block}
audio:not([controls]){display:none;height:0}
[hidden],template{display:none}
html{font-family:sans-serif;-ms-text-size-adjust:100%;-webkit-text-size-adjust:100%}
body{margin:0}
a{background:transparent}
a:focus{outline:thin dotted}
a:active,a:hover{outline:0}
h1{font-size:2em;margin:.67em 0}
abbr[title]{border-bottom:1px dotted}
b,strong{font-weight:bold}
dfn{font-style:italic}
hr{-moz-box-sizing:content-box;box-sizing:content-box;height:0}
mark{background:#ff0;color:#000}
code,kbd,pre,samp{font-family:monospace,serif;font-size:1em}
pre{white-space:pre-wrap}
q{quotes:"\201C" "\201D" "\2018" "\2019"}
small{font-size:80%}
sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}
sup{top:-0.5em}
sub{bottom:-0.25em}
img{border:0}
svg:not(:root){overflow:hidden}
figure{margin:0}
fieldset{border:1px solid #c0c0c0;margin:0 2px;padding:.35em .625em .75em}
legend{border:0;padding:0}
button,input,select,textarea{font-family:inherit;font-size:100%;margin:0}
button,input{line-height:normal}
button,select{text-transform:none}
button,html input[type="button"],input[type="reset"],input[type="submit"]{-webkit-appearance:button;cursor:pointer}
button[disabled],html input[disabled]{cursor:default}
input[type="checkbox"],input[type="radio"]{box-sizing:border-box;padding:0}
input[type="search"]{-webkit-appearance:textfield;-moz-box-sizing:content-box;-webkit-box-sizing:content-box;box-sizing:content-box}
input[type="search"]::-webkit-search-cancel-button,input[type="search"]::-webkit-search-decoration{-webkit-appearance:none}
button::-moz-focus-inner,input::-moz-focus-inner{border:0;padding:0}
textarea{overflow:auto;vertical-align:top}
table{border-collapse:collapse;border-spacing:0}

/* End of normalize.css */
/* Skyfield custom styles */
/* Colors darkened and lightened with http://gka.github.io/chroma.js/ */

@font-face {
    font-family: "Dosis";
    src: url(Dosis-Medium.ttf);
}
@font-face {
    font-family: "Gentium Custom";
    src: url(GentiumPlus-R-custom.woff);
}
@font-face {
    font-family: "Ubuntu Mono Custom";
    src: url(UbuntuMono-R-custom.woff);
}
body, body > img {
    display: block;
    margin-left: auto;
    margin-right: auto;
}
body {
    padding: 0 1em;
    line-height: 1.3;
    font-family: "Gentium Custom", sans-serif, serif;
    max-width: 36em;
}
body:after {
    display: block;
    padding-top: 0.4in;
    padding-bottom: 0.6in;
    text-align: center;
    content: "· · • · ·";
}
img {
    max-width: 100%;
}
h1, h2, h3 {
    font-family: "Dosis";
    color: #002664;
}
h3:before {
    content: "▸  ";
}
#skyfield h1 {
    margin-bottom: 0;
    height: 1px;
    text-indent: -9999px;  /* so screen reader will still read the title */
}
.logo {
    margin: 0 auto;
    display: block;
}
.headerlink {
    color: white;
    box-shadow: none;
}
:hover > .headerlink {
    color: #eee;
}
:hover > .headerlink:hover {
    color: #ddd;
    box-shadow: none;
}
.motto {
    margin-bottom: 1.75em;
    text-align: center;
}
.motto em {
    padding-bottom: 0.75em;
    border-bottom: 1px solid black;
}
.versionmodified {
    font-style: italic;
}
code, pre, tt, dt {
    font-family: "Ubuntu Mono Custom", monospace, serif;
    font-size: 0.83333333em;
}
code, pre, tt {
    background-color: #FDF6E3;
}
code, tt {
    border-radius: 3px;
    border: 1px solid #e3dcca;
    padding: 1px 2px 2px;
}
dt > code {
    font-size: 100%;
}
a code:hover, a tt:hover {
    border-color: #827c6b;
}
code.py-obj {
    border: none;
    border-radius: 0;
    background: none;
    font-family: "Gentium Custom", sans-serif, serif;
}
ul > li {
    margin-top: 0.75em;
    margin-bottom: 0.75em;
}
div.toctree-wrapper ul > li, div.contents ul > li {
    margin-top: 0em;
    margin-bottom: 0em;
}
dt {
    display: inline-block;
    padding-left: 4em;
    text-indent: -4em;
    color: #790000
}
dt em {
    font-style: normal;
}
dt code {
    border: none;
    padding: 0;
    background: none;
}
pre {
    border: 1px dashed #93A1A1;
    padding: 0.5em 0.8em;
    line-height: 1.25;
    overflow-wrap: break-word;
    font-size: 0.83333333em;
}
.highlight-none pre {
    border: none;
}
a {
    text-decoration: none;
    box-shadow: inset 0 -3px 0 white, inset 0 -4px 0 #ccc;
}
a:hover {
    box-shadow: inset 0 -3px 0 white, inset 0 -4px 0 black;
}
#api-reference table a {
    box-shadow: none;
}
#api-reference table a:hover {
    box-shadow: inset 0 -3px 0 white, inset 0 -4px 0 #ccc;
}
pre tt {
    padding: 0;
    font-size: 1.25em;
}
table[border="1"] {
    margin: 1em auto;
    border: none;
}
td {
    border: none;
    border-bottom: 1px solid #eee;
    padding: 0.2em 0.5em 0.2em 0;
}
tr:last-child td {
    border: none;
}
.viewcode-link {
    float: right;
    margin-left: 1em;
    color: #888;
}
dt:target {
   background-color: #f1d592;
}
@media screen and (min-width: 480px) {body {font-size: 1.125em;}}
@media screen and (min-width: 640px) {body {max-width: 42em;}}

/* To regenerate the following Pygments palette, run:
   pip install pygments-style-solarized
   pygmentize -S solarizedlight -f html > solarizedlight.css */

.hll { background-color: #ffffcc }
.c { color: #93a1a1; font-style: italic } /* Comment */
.err { color: #dc322f } /* Error */
.g { color: #657b83 } /* Generic */
.k { color: #859900 } /* Keyword */
.l { color: #657b83 } /* Literal */
.n { color: #586e75 } /* Name */
.o { color: #657b83 } /* Operator */
.x { color: #657b83 } /* Other */
.p { color: #657b83 } /* Punctuation */
.cm { color: #93a1a1; font-style: italic } /* Comment.Multiline */
.cp { color: #93a1a1; font-style: italic } /* Comment.Preproc */
.c1 { color: #93a1a1; font-style: italic } /* Comment.Single */
.cs { color: #93a1a1; font-style: italic } /* Comment.Special */
.gd { color: #657b83 } /* Generic.Deleted */
.ge { color: #657b83 } /* Generic.Emph */
.gr { color: #657b83 } /* Generic.Error */
.gh { color: #657b83 } /* Generic.Heading */
.gi { color: #657b83 } /* Generic.Inserted */
.go { color: #657b83 } /* Generic.Output */
.gp { color: #657b83 } /* Generic.Prompt */
.gs { color: #657b83 } /* Generic.Strong */
.gu { color: #657b83 } /* Generic.Subheading */
.gt { color: #657b83 } /* Generic.Traceback */
.kc { color: #859900 } /* Keyword.Constant */
.kd { color: #859900 } /* Keyword.Declaration */
.kn { color: #cb4b16 } /* Keyword.Namespace */
.kp { color: #cb4b16 } /* Keyword.Pseudo */
.kr { color: #859900 } /* Keyword.Reserved */
.kt { color: #859900 } /* Keyword.Type */
.ld { color: #657b83 } /* Literal.Date */
.m { color: #2aa198 } /* Literal.Number */
.s { color: #2aa198 } /* Literal.String */
.na { color: #657b83 } /* Name.Attribute */
.nb { color: #268bd2 } /* Name.Builtin */
.nc { color: #268bd2 } /* Name.Class */
.no { color: #b58900 } /* Name.Constant */
.nd { color: #cb4b16 } /* Name.Decorator */
.ni { color: #cb4b16 } /* Name.Entity */
.ne { color: #cb4b16 } /* Name.Exception */
.nf { color: #268bd2 } /* Name.Function */
.nl { color: #657b83 } /* Name.Label */
.nn { color: #b58900 } /* Name.Namespace */
.nx { color: #657b83 } /* Name.Other */
.py { color: #268bd2 } /* Name.Property */
.nt { color: #859900 } /* Name.Tag */
.nv { color: #cd4b16 } /* Name.Variable */
.ow { color: #859900 } /* Operator.Word */
.w { color: #fdf6e3 } /* Text.Whitespace */
.mf { color: #2aa198 } /* Literal.Number.Float */
.mh { color: #2aa198 } /* Literal.Number.Hex */
.mi { color: #2aa198 } /* Literal.Number.Integer */
.mo { color: #2aa198 } /* Literal.Number.Oct */
.sb { color: #2aa198 } /* Literal.String.Backtick */
.sc { color: #2aa198 } /* Literal.String.Char */
.sd { color: #2aa198 } /* Literal.String.Doc */
.s2 { color: #2aa198 } /* Literal.String.Double */
.se { color: #cb4b16 } /* Literal.String.Escape */
.sh { color: #2aa198 } /* Literal.String.Heredoc */
.si { color: #cb4b16 } /* Literal.String.Interpol */
.sx { color: #2aa198 } /* Literal.String.Other */
.sr { color: #2aa198 } /* Literal.String.Regex */
.s1 { color: #2aa198 } /* Literal.String.Single */
.ss { color: #2aa198 } /* Literal.String.Symbol */
.bp { color: #268bd2; font-weight: bold } /* Name.Builtin.Pseudo */
.vc { color: #268bd2 } /* Name.Variable.Class */
.vg { color: #268bd2 } /* Name.Variable.Global */
.vi { color: #268bd2 } /* Name.Variable.Instance */
.il { color: #2aa198 } /* Literal.Number.Integer.Long */
